//////////////////////////////////////////////////////////////////////////////
// GAMEPLAY.C                                                               //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <gb.h>
#include "gameplay.h"
#include "scroll.h"

//////////////////////////////////////////////////////////////////////////////
// variables                                                                //
//////////////////////////////////////////////////////////////////////////////
long dirx,diry;
unsigned long ballx,bally; // *16 for pixel, *256 for tile
//unsigned long ball2x,ball2y; // *16 for pixel, *256 for tile (multiplayer)
unsigned long scrollx,scrolly; // *16 for tile
unsigned int defaultbank;
unsigned int counter; // VBL counter
unsigned int status, lives, level;
unsigned int balltile;

extern char parallax[8][8][16]; // PARALLAX_BANK
extern int corner[4][16][16][2]; // {-sin,cos} ({0,x} if not blocked)
extern int hole[16][16][2];

//////////////////////////////////////////////////////////////////////////////
// LoadLevelData - Load level graphics etc.                                 //
//////////////////////////////////////////////////////////////////////////////
void LoadLevelData
(
	int mapbank, void *map,
	int tilebank, void *tiles, UWORD *tilepals,
	int spritebank, void *sprites, UWORD *spritepals
)
{
	// copy tilemap to RAM (attrmap stays in ROM, right after tilemap!)
	defaultbank = mapbank;
	SWITCH_ROM_MBC1(defaultbank);
	memcpy(&tilemap, map, 64*64L);

	// init tiles & palettes
	SWITCH_ROM_MBC1(tilebank);
	VBK_REG = 0; set_data((void*)0x8000, tiles, 64*64L);
	set_bkg_palette(0,8, tilepals);
	
	// init sprites & palettes
	SWITCH_ROM_MBC1(spritebank);
	VBK_REG = 1; set_data((void*)0x8000, sprites, 32*64L);
	set_sprite_palette(0,8, spritepals);
	// leave spritebank set to place items
}

//////////////////////////////////////////////////////////////////////////////
// sqr                                                                      //
//////////////////////////////////////////////////////////////////////////////
unsigned long sqr(long n)
{
	return n*n;
}

//////////////////////////////////////////////////////////////////////////////
// UpdateStatus - Update gamestatus                                         //
//////////////////////////////////////////////////////////////////////////////
void UpdateStatus()
{
	switch (status)
	{
		case S_APPEARING:
			if (counter & 0x3) break;
			if (++balltile == BALL) status = S_PLAYING;
			break;

		case S_DISAPPEARING:
			if (counter & 0x3) break;
			if (balltile > NOBALL)
				balltile--;
			else
				status = S_EXITLEVEL;
			break;
			
		case S_FALLING:
			if (counter & 0x3) break;
			if (--balltile <= NOBALL2)
			{
				PlaySound(SFX_BREAKGL);
				status = S_BREAKING;
			}
			break;
		
		case S_BREAKING:
			if (!SOUND_PLAYING)
			{
				if (--lives) // appear again
				{
					long x = (ballx - dirx)>>8;
					long y = (bally - diry)>>8;
					if (tilemap[y][x] <= 6) PUT_TILE(x,y,6);
					ballx = (x*16 + 6)*16L;
					bally = (y*16 + 6)*16L;
					dirx = diry = 0;
					status = S_APPEARING;
					balltile = NOBALL;
				}
				else // gameover
				{
					PlaySound(SFX_GAMEOVER);
					status = S_EXITLEVEL;
				}
			}
			break;
	}
}

//////////////////////////////////////////////////////////////////////////////
// TouchCorner                                                              //
//////////////////////////////////////////////////////////////////////////////
int TouchCorner(unsigned int cnr, int ix, int iy)
{
	if (tilemap[_bally+iy][_ballx+ix] >= 48) // diagonally blocked?
	if (tilemap[_bally][_ballx+ix] < 48) // left/right free?
	if (tilemap[_bally+iy][_ballx] < 48) // up/down free?
	{
		int *c = (int*)&corner[cnr][bally>>4&15][ballx>>4&15];
		if ((c[0] != 0) || (c[1] != 0))
		{
			long tmpdirx = (dirx*c[1] + diry*c[0]) / 64;
			diry = (-diry*c[1] + dirx*c[0]) / 64;
			dirx = tmpdirx;
			TouchBrick(_ballx+ix, _bally+iy, ix,iy); // touch brick!
			return 1;
		}
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// UpdateBall - Updates the ball position                                   //
//////////////////////////////////////////////////////////////////////////////
void UpdateBall(UBYTE joy)
{
	int fixx=0,fixy=0;

	if (status != S_PLAYING) return;
	
	if (joy & J_UP) diry-=3;
	if (joy & J_DOWN) diry+=3;
	if (joy & J_LEFT) dirx-=3;
	if (joy & J_RIGHT) dirx+=3;

	ballx += dirx;
	bally += diry;

/*	//(multiplayer)
	if (sqr((ballx>>4) - (ball2x>>4)) + sqr((bally>>4) - (ball2y>>4)) <= 144L)
	{
		int *c = (int*)&corner[0][((ball2y>>4) - (bally>>4))&15][((ball2x>>4) - (ballx>>4))&15];
		long tmpdirx = (dirx*c[1] + diry*c[0]) / 64;
		diry = (-diry*c[1] + dirx*c[0]) / 64;
		dirx = tmpdirx;
		fixx = dirx*4;
		fixy = diry*4;
	}*/

	if ( TouchCorner(0, -1, -1)
	|| TouchCorner(1, +1, -1)
	|| TouchCorner(2, +1, +1)
	|| TouchCorner(3, -1, +1) )
	{
		fixx = dirx;
		fixy = diry;
	}
	else
	{
		if (dirx < 0) if (TouchBrick((ballx-6*16)>>8, _bally, -1,0))
		{
			dirx = -dirx;
			fixx = dirx;
		}

		if (diry < 0) if (TouchBrick(_ballx, (bally-6*16)>>8, 0,-1))
		{
			diry = -diry;
			fixy = diry;
		}

		if (dirx > 0) if (TouchBrick((ballx+6*16)>>8, _bally, +1,0))
		{
			dirx = -dirx;
			fixx = dirx;
		}

		if (diry > 0) if (TouchBrick(_ballx, (bally+6*16)>>8, 0,+1))
		{
			diry = -diry;
			fixy = diry;
		}
	}

	ballx += fixx;
	bally += fixy;

	// check floor
	CheckFloor(_ballx, _bally); //, sqr(dirx) + sqr(diry));
}

//////////////////////////////////////////////////////////////////////////////
// NextFrame - Next frame (call immediately after v-blank)                  //
//////////////////////////////////////////////////////////////////////////////
void NextFrame()
{
	unsigned int oldbank = defaultbank;
	unsigned int x,y;
	SCY_REG = scrolly;
	SCX_REG = scrollx;
	counter++;
	
	// parallax scrolling tile
	SWITCH_BANK(PARALLAX_BANK);
	x = (SCX_REG>>1)&7;
	y = 7-(SCY_REG>>1)&7;
	set_bkg_data(0,1, (void*)&parallax[y][x][0]);
	SWITCH_BANK(oldbank);
}

//////////////////////////////////////////////////////////////////////////////
// PrepareNextFrame - Prepare next frame                                    //
//////////////////////////////////////////////////////////////////////////////
void PrepareNextFrame()
{
	int x,y;

	// calculate scroll increments
	y = (bally>>4)-72 - scrolly;
	CLIP(y, -48, 48); y >>= 3; // -16..16
	x = (ballx>>4)-80 - scrollx;
	CLIP(x, -48, 48); x >>= 3; // -16..16

	// scroll
	scrollx += x;
	if ((counter & 1) != 0)
	{
		if (y > 0) ScrollVert(11, (scrollx>>4), (scrolly>>4)+9); // down
		if (y < 0) ScrollVert(11, (scrollx>>4), (scrolly>>4)-1); // up
	}
	scrolly += y;
	if ((counter & 1) == 0)
	{
		if (x > 0) ScrollHorz(11, (scrollx>>4)+10, (scrolly>>4)); // right
		if (x < 0) ScrollHorz(11, (scrollx>>4)-0, (scrolly>>4)); // left
	}

	// update ball sprite
	sprites16x16[0].x = (ballx>>4)+2;
	sprites16x16[0].y = (bally>>4)+10;
	sprites16x16[0].firsttile = balltile<<2;

/*	// update opponent ball sprite (multiplayer)
	sprites16x16[1].x = (ball2x>>4)+2;
	sprites16x16[1].y = (ball2y>>4)+10;
	sprites16x16[1].firsttile = 15<<2;*/

	// draw sprites
	DrawSprites();
}

//////////////////////////////////////////////////////////////////////////////
// Init                                                                     //
//////////////////////////////////////////////////////////////////////////////
void Init()
{
	disable_interrupts();
	IE_REG |= VBL_IFLAG | TIM_IFLAG;
	TAC_REG = 0x04; // 4KHz
	TMA_REG = 256-16; // 256Hz
	InitSound();
	SPRITES_8x16;
	SHOW_BKG; // enable sprite priority setting on GBC
}

//////////////////////////////////////////////////////////////////////////////
// CheckSprite                                                              //
//////////////////////////////////////////////////////////////////////////////
void CheckSprite(unsigned int spritetile, Sprite *s)
{
	switch (spritetile)
	{
		case KEY:
			KILL_SPRITE(s);
			PlaySound(SFX_PICKUP);
			break;
		
		case HAMMER:
			KILL_SPRITE(s);
			PlaySound(SFX_PICKUP);
			break;
	}
}

//////////////////////////////////////////////////////////////////////////////
// CheckFloor                                                               //
//////////////////////////////////////////////////////////////////////////////
void CheckFloor(unsigned int x, unsigned int y/*, unsigned long speed*/)
{
	unsigned char tile = tilemap[y][x];
	switch (tile)
	{
		case EMPTY: // fall down
			balltile = BALL2;
			status = S_FALLING;
			return;

		case UP:
			diry-=5;
			break;
			
		case RIGHT:
			dirx+=5;
			break;
			
		case DOWN:
			diry+=5;
			break;
			
		case LEFT:
			dirx-=5;
			break;
			
		case BRKCON1: // broken concrete
		case BRKCON2:
		case BRKCON3:
		case BRKCON4:
		case BRKCON5:
		case BRKCON6:
			if ((counter&0xF) == 0)
			{
				PlaySound(SFX_CRACK);
				PUT_TILE(x,y, tile-1);
			}
			break;
			
		case HOLE:
			dirx += hole[bally>>4&15][ballx>>4&15][0];
			diry += hole[bally>>4&15][ballx>>4&15][1];
			break;
			
		case HILL:
			dirx -= hole[bally>>4&15][ballx>>4&15][0];
			diry -= hole[bally>>4&15][ballx>>4&15][1];
			break;
	}
	
	// decrease speed due friction
	dirx = (dirx<<5)/33;
	diry = (diry<<5)/33;
}

//////////////////////////////////////////////////////////////////////////////
// TouchBrick - Touch brick if blocked                                      //
//////////////////////////////////////////////////////////////////////////////
int TouchBrick(unsigned int x, unsigned int y, int ix, int iy)
{
	unsigned long speed;
	if (tilemap[y][x] < 48) return 0;
	speed = (ix ? sqr(dirx) : 0) + (iy ? sqr(diry) : 0);

	switch (tilemap[y][x])
	{
		case EXIT:
			if ((ix==0) && (iy==0)) break;
			if ((ix!=0) && (iy!=0)) break;
			if (ix==0)
			{
				if ((*(unsigned int*)&ballx) < 80) break;
				if ((*(unsigned int*)&ballx) > 176) break;
			}
			if (iy==0)
			{
				if ((*(unsigned int*)&bally) < 80) break;
				if ((*(unsigned int*)&bally) > 176) break;
			}
			PlaySound(SFX_ENDLEVEL);
			status = S_DISAPPEARING;
			break;

/*		case SWITCH_OFF:
			PUT_TILE(x,y, SWITCH_ON);
			break;*/
			
		default:
			if (speed > 200)
			{
				PlaySound(SFX_BOUNCE);
/*				// damn... can't override bkg tile priority
				Sprite *s;
				s = NewSprite8x16();
				s->x = ballx>>4;// x*16L;
				s->y = bally>>4; //y*16L;
				s->firsttile = 17*4 + 2;
				s->prop = 8+1;*/
			}
			break;
	}

	return 1;
}

//////////////////////////////////////////////////////////////////////////////
// level data                                                               //
//////////////////////////////////////////////////////////////////////////////
#include "tiles1.h"
#include "sprites1.h"

UWORD tiles1pals[] = CGBPALS(tiles1);
UWORD sprites1pals[] = CGBPALS(sprites1);

//////////////////////////////////////////////////////////////////////////////
// LoadLevel                                                                //
//////////////////////////////////////////////////////////////////////////////
void LoadLevel(unsigned int level)
{
	int y;
	
	disable_interrupts();
	LCDC_REG |= 0x10;
	SHOW_SPRITES;
	ClearSprites();

	switch (level)
	{
		case 1:
		{
			extern unsigned char map1[][64];
			LoadLevelData(MAP1, TILES1, SPRITES1);
			PlaceBallSprite16x16(7,6, NOBALL, sprites1CGB[NOBALL]);
			
/*			PlaceNewSprite16x16(8,8, 15, sprites1CGB[15]); // (multiplyer)
			ball2x = (8*16L + 6)*16L;
			ball2y = (8*16L + 6)*16L;*/
			
			PlaceNewSprite16x16(7,11, HAMMER, sprites1CGB[HAMMER]);
			PlaceNewSprite16x16(10,10, KEY, sprites1CGB[KEY]);
			break;
		}
		case 2:
		{
			extern unsigned char map2[][64];
			LoadLevelData(MAP2, TILES1, SPRITES1);
			PlaceBallSprite16x16(7,6, NOBALL, sprites1CGB[NOBALL]);
			break;
		}
	}

	SCX_REG = scrollx = (ballx>>4)-80;
	SCY_REG = scrolly = (bally>>4)-72;
	dirx = diry = 0;
	status = S_APPEARING;
	lives = 3;
	counter = 0;
	
	// draw what's on screen
	SWITCH_ROM_MBC1(defaultbank);
	for (y=0; y<10; y++) ScrollVert(11, (scrollx>>4), (scrolly>>4) + y);

	enable_interrupts();
}
