//////////////////////////////////////////////////////////////////////////////
// GAMEPLAY.H                                                               //
//////////////////////////////////////////////////////////////////////////////
#ifndef _GAMEPLAY_H
#define _GAMEPLAY_H

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include "sprites.h"
#include "sound.h"

//////////////////////////////////////////////////////////////////////////////
// defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#define LEVELS		2

// banks & pointers
#define PARALLAX_BANK 5
#define TILES1		4, tiles1, tiles1pals
#define SPRITES1	4, sprites1, sprites1pals
#define	MAP1		4, map1
#define	MAP2		6, map2

// GBTD palettes to array converter
#define CGBPAL(id,pal) id##CGBPal##pal##c0,id##CGBPal##pal##c1,id##CGBPal##pal##c2,id##CGBPal##pal##c3
#define CGBPALS(id) {CGBPAL(id,0),CGBPAL(id,1),CGBPAL(id,2),CGBPAL(id,3),CGBPAL(id,4),CGBPAL(id,5),CGBPAL(id,6),CGBPAL(id,7)}

// ball position -> map position (faster than /256 or >>8)
#define _ballx (*((char*)&ballx+1))
#define _bally (*((char*)&bally+1))

// status
enum {S_FALLING, S_BREAKING, S_APPEARING, S_PLAYING, S_DISAPPEARING, S_EXITLEVEL};

//////////////////////////////////////////////////////////////////////////////
// SWITCH_BANK                                                              //
//////////////////////////////////////////////////////////////////////////////
#define SWITCH_BANK(n) \
{ \
	SWITCH_ROM_MBC1(defaultbank = (n)); \
}

//////////////////////////////////////////////////////////////////////////////
// CLIP - Clips a variable to a min. & a max. value                         //
//////////////////////////////////////////////////////////////////////////////
#define CLIP(n,min,max) \
{ \
	if ((n) < (min)) (n) = (min); \
	if ((n) > (max)) (n) = (max); \
}

//////////////////////////////////////////////////////////////////////////////
// PUT_TILE - Change tile in map and update it on screen                    //
//////////////////////////////////////////////////////////////////////////////
#define PUT_TILE(x,y,tile) \
{ \
	tilemap[y][x] = tile; \
	if ((x >= (scrollx>>4)) && (x <= (scrollx>>4)+10) \
	 && (y >= (scrolly>>4)) && (y <= (scrolly>>4)+9)) \
		ScrollVert(1, x,y); \
}

//////////////////////////////////////////////////////////////////////////////
// variables                                                                //
//////////////////////////////////////////////////////////////////////////////
extern long dirx,diry;
extern unsigned long ballx,bally; // *16 for pixel, *256 for tile
extern unsigned long ball2x,ball2y; // *16 for pixel, *256 for tile (multiplayer)
extern unsigned long scrollx,scrolly; // *16 for tile
extern unsigned int defaultbank;
extern unsigned int counter; // VBL counter
extern unsigned int status, lives, level;
extern unsigned int balltile;

//////////////////////////////////////////////////////////////////////////////
// tile defines                                                             //
//////////////////////////////////////////////////////////////////////////////
#define EMPTY	0
#define BRKCON1	1
#define BRKCON2	2
#define BRKCON3	3
#define BRKCON4	4
#define BRKCON5	5
#define BRKCON6	6
#define HOLE	10
#define HILL	11
#define UP		12
#define RIGHT	13
#define DOWN	14
#define LEFT	15

//////////////////////////////////////////////////////////////////////////////
// wall defines                                                             //
//////////////////////////////////////////////////////////////////////////////
#define EXIT	54

//////////////////////////////////////////////////////////////////////////////
// sprite defines                                                           //
//////////////////////////////////////////////////////////////////////////////
#define NOBALL2	0
#define	BALL2	7
#define NOBALL	8
#define	BALL	15
#define HAMMER	18
#define KEY		19

//////////////////////////////////////////////////////////////////////////////
// prototypes                                                               //
//////////////////////////////////////////////////////////////////////////////
void UpdateStatus();
void UpdateBall(UBYTE joy);
void NextFrame();
void PrepareNextFrame();
void Init();
void CheckSprite(unsigned int spritetile, Sprite *s);
void CheckFloor(unsigned int x, unsigned int y/*, unsigned long speed*/);
int TouchBrick(unsigned int x, unsigned int y, int ix, int iy);
void LoadLevel(unsigned int level);

#endif // _GAMEPLAY_H
