;////////////////////////////////////////////////////////////////////////////;
;/ SCROLL.S - Metatile scrolling routines for 64x64 maps                    /;
;////////////////////////////////////////////////////////////////////////////;
;tiles must be at 0xC800 (copied from 0x4000, but can be modified now)
;attributes must be at 0x5000 (fixed, so blocks cannot be moved yet)

;notes when making changes:
;	_ScrollHorz and _ScrollVert are identical !!!
;	.CopyAttrXXXX is the same as .CopyTileXXXX except for the tilenumber calculations
;	.ParallaxTilesXXXX is without three extra "inc a"'s

	.include "\dev\gb\lib\global.s"

_tilemap == 0xC800
_attrmap == 0x5000

;////////////////////////////////////////////////////////////////////////////;
	.area	_BSS
;////////////////////////////////////////////////////////////////////////////;

.count:				;metatiles to copy when scrolling
	.ds 1

;////////////////////////////////////////////////////////////////////////////;
	.area	_CODE
;////////////////////////////////////////////////////////////////////////////;

;////////////////////////////////////////////////////////////////////////////;
;/ .CalcSource - Calculates map source address                              /;
;////////////////////////////////////////////////////////////////////////////;
;	enter:		hl = &(x,y)
;				b = 4 MSB of &map
;	return:		de = &map[y][x]

.CalcSource:
	ld	a, (hl+)	;________00xxxxxx
	sla	a			;________0xxxxxx0
	sla	a			;________xxxxxx00
	ld	d, (hl)		;00yyyyyy________
	srl	d			;000yyyyyY_______
	rra				;________yxxxxxx0
	srl	d			;0000yyyyY_______
	rra				;________yyxxxxxx
	ld	e, a		;________yyxxxxxx
	ld	a, b		;eeee0000________
	or	a, d		;eeeeyyyy________
	ld	d, a		;eeeeyyyy________
					;////////////////
					;eeeeyyyyyyxxxxxx (64*64 map)
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .CalcDest - Calculates tilemap destination address                       /;
;////////////////////////////////////////////////////////////////////////////;
;	enter:		hl = &(x,y)+1
;	return:		hl = &tilemap[y][x]

.CalcDest:
	ld	a, (hl-)	;________00yyyyyy
	and	a, #0x0F	;________0000yyyy
	ld	c, a		;________0000yyyy
	swap c			;________yyyy0000
	sla	c			;_______Yyyy00000
	ld	b, #0x98>>2	;00100110________
	rl	b			;0100110y________
	sla	c			;_______Yyy000000
	rl	b			;100110yy________
	
	ld	a, (hl)		;________00xxxxxx
	and	a, #0x0F	;________0000xxxx
	sla	a			;________000xxxx0
	or	a, c		;________yy0xxxx0

	ld	h, b		;100110yy________
	ld	l, a		;________yy0xxxx0
					;////////////////
					;100110yyyy0xxxx0 (32*32 map, >0x9800)
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _ScrollVert (int count=11, int x, int y) - Scroll vertically             /;
;////////////////////////////////////////////////////////////////////////////;
.CopyAttr	= .CopyAttrVert
.CopyTiles	= .CopyTilesVert

_ScrollVert::
	push bc
	
	lda	hl, 4(sp)
	ld	a, (hl+)
	ld	(.count), a

	ld	b, #>_attrmap	;0101yyyyyyxxxxxx (>0x5000)
	call .CalcSource
	push de			;save source address
	call .CalcDest
	push hl			;save destination address
	
	ld	a, #1		;attribute map
	ldh	(.vbk), a
	ld	a, (.count)
	ld	c, a
	call .CopyAttr
	
	pop	hl			;restore destination address
	pop	de			;restore source address
	ld	a, d
	and	a, #0x0F	;0000yyyyyyxxxxxx
	add	a, #>_tilemap	;1101yyyyyyxxxxxx (>0xC800)
	ld	d, a

	xor	a, a		;tile map
	ldh	(.vbk), a
	ld	a, (.count)
	ld	c, a
	call .CopyTiles

	pop	bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _ScrollHorz (int count=10, int x, int y) - Scroll horizontally           /;
;////////////////////////////////////////////////////////////////////////////;
.CopyAttr	= .CopyAttrHorz
.CopyTiles	= .CopyTilesHorz

_ScrollHorz::
	push bc
	
	lda	hl, 4(sp)
	ld	a, (hl+)
	ld	(.count), a
	
	ld	b, #>_attrmap	;0101yyyyyyxxxxxx (>0x5000)
	call .CalcSource
	push de			;save source address
	call .CalcDest
	push hl			;save destination address
	
	ld	a, #1		;attribute map
	ldh	(.vbk), a
	ld	a, (.count)
	ld	c, a
	call .CopyAttr
	
	pop	hl			;restore destination address
	pop	de			;restore source address
	ld	a, d
	and	a, #0x0F	;0000yyyyyyxxxxxx
	add	a, #>_tilemap	;1101yyyyyyxxxxxx (>0xC800)
	ld	d, a

	xor	a, a		;tile map
	ldh	(.vbk), a
	ld	a, (.count)
	ld	c, a
	call .CopyTiles

	pop	bc
	ret

;////////////////////////////////////////////////////////////////////////////;

;Vertical scroll drawing
;
;     012345..
;     ------..
; 0 | 020202..
;32 | 131313..

;////////////////////////////////////////////////////////////////////////////;
;/ .CopyTilesVert - Vertical scroll tile loop                               /;
;////////////////////////////////////////////////////////////////////////////;
.CopyTilesVert:

2$:					;wait for vram busy
	ldh	a, (.STAT)
	and	a, #0x02
	jr	z, 2$
1$:					;wait for vram
	ldh	a, (.STAT)
	and	a, #0x02
	jr	nz, 1$

	ld	a, (de)		;tilenr*4
	inc	de
	sla	a
	sla	a
	ld	(hl), a		;100110yyyy0xxxx0
	set	5, l		;100110yyyy1xxxx0	
		cp	a, #0
		jr	z, .ParallaxTilesVert	
	inc	a
	ld	(hl+), a	;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	inc	a
	ld	(hl), a		;100110yyyy0xxxx1
	set	5, l		;100110yyyy1xxxx1
	inc	a
	ld	(hl), a		;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	inc	l			;100110yyyy?xxxx0
	res	5, l		;100110yyyy0xxxx0
	
	dec	c
	jr	nz, .CopyTilesVert
	
	ret

.ParallaxTilesVert:
	ld	(hl+), a	;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	ld	(hl), a		;100110yyyy0xxxx1
	set	5, l		;100110yyyy1xxxx1
	ld	(hl), a		;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	inc	l			;100110yyyy?xxxx0
	res	5, l		;100110yyyy0xxxx0
	
	dec	c
	jr	nz, .CopyTilesVert
	
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .CopyAttrVert - Vertical scroll attribute loop                           /;
;////////////////////////////////////////////////////////////////////////////;
.CopyAttrVert:

2$:					;wait for vram
	ldh	a, (.STAT)
	and	a, #0x02
	jr	z, 2$
1$:					;wait for vram
	ldh	a, (.STAT)
	and	a, #0x02
	jr	nz, 1$
	
	ld	a, (de)		;attr
	inc	de
	ld	(hl), a		;100110yyyy0xxxx0
	set	5, l		;100110yyyy1xxxx0
	ld	(hl+), a	;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	ld	(hl), a		;100110yyyy0xxxx1
	set	5, l		;100110yyyy1xxxx1
	ld	(hl), a		;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	inc	l			;100110yyyy?xxxx0
	res	5, l		;100110yyyy0xxxx0
	
	dec	c
	jr	nz, .CopyAttrVert
	
	ret

;////////////////////////////////////////////////////////////////////////////;

;Horizontal scroll drawing
;
;     01
;     --
; 0 | 02
;32 | 13
;64 | 02
;96 | 13
;:: : ::

;////////////////////////////////////////////////////////////////////////////;
;/ .CopyTilesHorz - Horizontal scroll tile loop                             /;
;////////////////////////////////////////////////////////////////////////////;
.CopyTilesHorz:

2$:					;wait for vram busy
	ldh	a, (.STAT)
	and	a, #0x02
	jr	z, 2$
1$:					;wait for vram
	ldh	a, (.STAT)
	and	a, #0x02
	jr	nz, 1$
	
	ld	a, (de)		;tilenr*4
	push hl
	ld	hl, #64
	add	hl, de
	ld	d, h
	ld	e, l
	pop	hl
	
	sla	a
	sla	a
	ld	(hl), a		;100110yyyy0xxxx0
	set	5, l		;100110yyyy1xxxx0
		cp	a, #0
		jr	z, .ParallaxTilesHorz
	inc	a
	ld	(hl+), a	;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	inc	a
	ld	(hl), a		;100110yyyy0xxxx1
	set	5, l		;100110yyyy1xxxx1
	inc	a
	ld	(hl-), a	;100110yyyy1xxxx0
	
	ld	a, c
	ld	bc, #32
	add	hl, bc		;10011?yyyy0xxxx0
	ld	c, a
	
	res	2, h		;100110yyyy0xxxx0
	
	dec	c
	jr	nz, .CopyTilesHorz
	
	ret

.ParallaxTilesHorz:
	ld	(hl+), a	;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	ld	(hl), a		;100110yyyy0xxxx1
	set	5, l		;100110yyyy1xxxx1
	ld	(hl-), a	;100110yyyy1xxxx0
	
	ld	a, c
	ld	bc, #32
	add	hl, bc		;10011?yyyy0xxxx0
	ld	c, a
	
	res	2, h		;100110yyyy0xxxx0
	
	dec	c
	jr	nz, .CopyTilesHorz
	
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .CopyAttrHorz - Vertical scroll attribute loop                           /;
;////////////////////////////////////////////////////////////////////////////;
.CopyAttrHorz:

2$:					;wait for vram
	ldh	a, (.STAT)
	and	a, #0x02
	jr	z, 2$
1$:					;wait for vram
	ldh	a, (.STAT)
	and	a, #0x02
	jr	nz, 1$
	
	ld	a, (de)		;attr
	push hl
	ld	hl, #64
	add	hl, de
	ld	d, h
	ld	e, l
	pop	hl
	
	ld	(hl), a		;100110yyyy0xxxx0
	set	5, l		;100110yyyy1xxxx0
	ld	(hl+), a	;100110yyyy1xxxx1
	res	5, l		;100110yyyy0xxxx1
	ld	(hl), a		;100110yyyy0xxxx1
	set	5, l		;100110yyyy1xxxx1
	ld	(hl-), a	;100110yyyy1xxxx0
	
	ld	a, c
	ld	bc, #32
	add	hl, bc		;10011?yyyy0xxxx0
	ld	c, a
	
	res	2, h		;100110yyyy0xxxx0
	
	dec	c
	jr	nz, .CopyAttrHorz
	
	ret
