//////////////////////////////////////////////////////////////////////////////
// GENERATE.CPP                                                             //
//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <math.h>

//////////////////////////////////////////////////////////////////////////////
// MakeHole                                                                 //
//////////////////////////////////////////////////////////////////////////////
int MakeHole()
{
	FILE* fo = fopen("../tab/hole.c", "wt");
	if (!fo) { printf("Error writing file!\n"); return 1; }
	fprintf(fo, "//////////////////////////////////////////////////////////////////////////////\n");
	fprintf(fo, "// HOLE.C                                                                   //\n");
	fprintf(fo, "//////////////////////////////////////////////////////////////////////////////\n\n");
	fprintf(fo, "int hole[16][16][2] =\n{\n");
	for (int y=0; y<16; y++)
	{
		double y2 = (double)(int)(y-7.5);
		for (int x=0; x<16; x++)
		{
			double x2 = (double)(int)(x-7.5);
			fprintf(fo, "%+2.0f,", ( -cos(M_PI*y2/15) * sin(M_PI*x2/15) ) * 10 ); // x
			fprintf(fo, "%+2.0f,  ", ( -sin(M_PI*y2/15) * cos(M_PI*x2/15) ) * 10 ); // y
		}
		fprintf(fo, "\n");
	}
	fprintf(fo, "};\n");
	fclose(fo);
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// MakeCorner                                                               //
//////////////////////////////////////////////////////////////////////////////
int MakeCorner()
{
	FILE* fo = fopen("../tab/corner.c", "wt");
	if (!fo) { printf("Error writing file!\n"); return 1; }
	fprintf(fo, "//////////////////////////////////////////////////////////////////////////////\n");
	fprintf(fo, "// CORNER.C                                                                 //\n");
	fprintf(fo, "//////////////////////////////////////////////////////////////////////////////\n\n");
	fprintf(fo, "int corner[4][16][16][2] =\n{\n");

	for (int y=0; y<16; y++) // to upper left (-1,-1)
	{
		double y2 = (double)y+1;
		for (int x=0; x<16; x++)
		{
			if (x*x+y*y < 6*6)
			{
				double x2 = (double)x+1;
				double angle = atan(x2/y2)*2;
				fprintf(fo, "%+2.0f,", -sin(angle)*64);
				fprintf(fo, "%+2.0f,  ", cos(angle)*64);
			}
			else
			{
				fprintf(fo, "%+2.0f,", 0.0);
				fprintf(fo, "%+2.0f,  ", 0.0);
			}
		}
		fprintf(fo, "\n");
	}

	for (y=0; y<16; y++) // to upper right (+1,-1)
	{
		double y2 = (double)y+1;
		for (int x=15; x>=0; x--)
		{
			if (x*x+y*y < 6*6)
			{
				double x2 = (double)x+1;
				double angle = atan(x2/y2)*2;
				fprintf(fo, "%+2.0f,", sin(angle)*64);
				fprintf(fo, "%+2.0f,  ", cos(angle)*64);
			}
			else
			{
				fprintf(fo, "%+2.0f,", 0.0);
				fprintf(fo, "%+2.0f,  ", 0.0);
			}
		}
		fprintf(fo, "\n");
	}

	for (y=15; y>=0; y--) // to lower right (+1,+1)
	{
		double y2 = (double)y+1;
		for (int x=15; x>=0; x--)
		{
			if (x*x+y*y < 6*6)
			{
				double x2 = (double)x+1;
				double angle = atan(x2/y2)*2;
				fprintf(fo, "%+2.0f,", -sin(angle)*64);
				fprintf(fo, "%+2.0f,  ", cos(angle)*64);
			}
			else
			{
				fprintf(fo, "%+2.0f,", 0.0);
				fprintf(fo, "%+2.0f,  ", 0.0);
			}
		}
		fprintf(fo, "\n");
	}

	for (y=15; y>=0; y--) // to lower left (-1,+1)
	{
		double y2 = (double)y+1;
		for (int x=0; x<16; x++)
		{
			if (x*x+y*y < 6*6)
			{
				double x2 = (double)x+1;
				double angle = atan(x2/y2)*2;
				fprintf(fo, "%+2.0f,", sin(angle)*64);
				fprintf(fo, "%+2.0f,  ", cos(angle)*64);
			}
			else
			{
				fprintf(fo, "%+2.0f,", 0.0);
				fprintf(fo, "%+2.0f,  ", 0.0);
			}
		}
		fprintf(fo, "\n");
	}

	fprintf(fo, "};\n");
	fclose(fo);
	return 0;
}

/*        cos=-1 sin=0
  180| <= x = -1*x +  0*y
     |    y =  1*y +  0*x
0   /     cos=0 -sin=-1
---~ <=== x =  0*x + -1*y
^         y =  0*y + -1*x
|         cos=1 sin=0
 \======= x =  1*x +  0*y
          y = -1*y +  0*x

x =   x*cos + y*-sin
y = - y*cos + x*-sin 
*/

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main()
{
	if (MakeHole()) return 1;
	if (MakeCorner()) return 1;
	return 0;
}