//////////////////////////////////////////////////////////////////////////////
// PARALLAX.CPP                                                             //
//////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <string.h>

#define H 8
#define W 8

//////////////////////////////////////////////////////////////////////////////
// typedefs                                                                 //
//////////////////////////////////////////////////////////////////////////////
typedef struct
{
	char id[2]; // "BM"
	char n1[0x10];
	long width, height;
	char n2[2];
	short bpp, compression;
	char n3[22];
	char pal[256][4];
	char data[H][W];
} BMP;

//////////////////////////////////////////////////////////////////////////////
// defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#if sizeof(BMP) != 0x36 + 256*4 + H*W
	#error Invalid structure size!
#endif

#define ALIGN4(n) (((n)+3)/4*4)

BMP bmp;
FILE *fo;

//////////////////////////////////////////////////////////////////////////////
// ReadBMP                                                                  //
//////////////////////////////////////////////////////////////////////////////
int ReadBMP(char *filename)
{
	FILE *fi = fopen(filename, "rb");
	if (!fi) { printf("Input file error!\n"); return 1; }
	fread(&bmp, sizeof(BMP), 1, fi);
	if ((bmp.id[0] != 'B') || (bmp.id[1] != 'M')) { printf("Not a BMP-file!\n"); return 1; }
	if (bmp.bpp != 8) { printf("8 bits per pixel needed!\n"); return 1; }
	if (bmp.compression) { printf("No compression allowed!\n"); return 1; }
	if ((bmp.width != W) || (bmp.height != H)) { printf("Invalid image size!\n"); return 1; }
	fclose(fi);
	return 0;
}

/*
//////////////////////////////////////////////////////////////////////////////
// Part8x16                                                                 //
//////////////////////////////////////////////////////////////////////////////
void Part8x16(int ox, int oy, int sx, int sy)
{
	for (int y=0; y<16; y++)
	{
		unsigned char l=0,h=0;
		for (int x=7; x>=0; x--)
		{
			l = (l<<1) + ((bmp.data[32-((y+oy+sy)&31)][(x+ox+sx)&31] >> 0) & 1);
			h = (h<<1) + ((bmp.data[32-((y+oy+sy)&31)][(x+ox+sx)&31] >> 1) & 1);
		}
		fputc(l, fo);
		fputc(h, fo);
	}
}
*/

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	if (argc < 2) { printf("%s <image.bmp>\n", argv[0]); return 1; }

	if (ReadBMP(argv[1])) { printf("Error reading input file!\n"); return 1; }

	char s[256]; strcpy(s, argv[1]);
	char *p = strrchr(s, '.'); if (p) *p = 0; // remove extension
	strcat(s, ".bin"); // add .bin extension
	fo = fopen(s, "wb");
	if (!fo) { printf("Error creating output file!\n"); return 1; }

	for (int sy=0; sy<8; sy++)
	{
		for (int sx=0; sx<8; sx++)
		{
			for (int y=0; y<8; y++)
			{
				unsigned char l=0,h=0;
				for (int x=7; x>=0; x--)
				{
					l = (l<<1) + ((bmp.data[7-((y+sy)&7)][(x+sx)&7] >> 0) & 1);
					h = (h<<1) + ((bmp.data[7-((y+sy)&7)][(x+sx)&7] >> 1) & 1);
				}
				fputc(l, fo);
				fputc(h, fo);
			}

			/*Part8x16(0,0,sx,sy);
			Part8x16(8,0,sx,sy);
			Part8x16(0,16,sx,sy);
			Part8x16(8,16,sx,sy);
			Part8x16(16,0,sx,sy);
			Part8x16(24,0,sx,sy);
			Part8x16(16,16,sx,sy);
			Part8x16(24,16,sx,sy);*/
		}
	}

	fclose(fo);
	return 0;
}