#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <math.h>

typedef unsigned char BYTE;
typedef unsigned int WORD;
typedef unsigned long DWORD;

typedef struct
{
	BYTE riff_id[4]; // "RIFF"
	DWORD riff_len;

	BYTE wave_id[4]; // "WAVE"

	BYTE fmt_id[4]; // "fmt "
	DWORD fmt_len;
	WORD fmt_tag;
	WORD channels; // 1=mono, 2=stereo
	DWORD samplerate;
	DWORD samplerate2;
	WORD bytes_per_sample;
	WORD bits; // 8/16 bits

	BYTE data_id[4]; // "data"
	DWORD data_len;
} WAVE;

#if sizeof(WAVE) != 0x2C
	#error Size of structure WAVE is invalid!
#endif

#define BUFSAMPLES 4096

unsigned char wavbuf[BUFSAMPLES]; // 8 bit
unsigned char gbbuf[BUFSAMPLES/2]; // 4 bit
unsigned char rawbuf[BUFSAMPLES]; // 8 bit

double error=0.0;
double sample;
double wow[32];
double gb;

int main(int argc, char *argv[])
{
	if (argc < 2) { printf("%s [input.wav]\n", argv[0]); return 1; }
	FILE *fwav = fopen(argv[1], "rb");
	if (!fwav) { printf("Input error!\n"); return 1; }
	WAVE wave;
	fread(&wave, sizeof(WAVE), 1, fwav);
	if (strncmp(wave.riff_id, "RIFF", 4) != 0) { printf("RIFF error!\n"); return 1; };
	if (strncmp(wave.fmt_id, "fmt ", 4) != 0) { printf("FMT error!\n"); return 1; };
	if (strncmp(wave.data_id, "data", 4) != 0) { printf("DATA error!\n"); return 1; };

	char s[256]; strcpy(s, argv[1]);
	char *p = strrchr(s, '.'); if (p) *p = 0; // remove extension
	strcat(s, ".bin"); // add .bin extension
	FILE *fgb = fopen(s, "wb");
	if (!fgb) { printf("Output error!\n"); return 1; }
	int blocks = 0;
	fwrite(&blocks, 2, 1, fgb);

	//FILE *fraw = fopen("wav2bin.raw", "wb");

	while (wave.data_len > 0)
	{
		long read;
		if (wave.data_len >= BUFSAMPLES)
		{
			read = fread(wavbuf, 1, BUFSAMPLES, fwav); // 8 bit
			wave.data_len -= BUFSAMPLES;
		}
		else
		{
			read = fread(wavbuf, 1, wave.data_len, fwav); // 8 bit
			wave.data_len = 0;
		}

		for (long i=0; i<read; i++)
		{
			//lastsample = (lastsample*30.0 + inputsample*80.0) / 100.0;
/*			wow[i&3] = wavbuf[i];
			double total = 0.0;
			for (int n=0; n<4; n++) { total += wow[n]; wow[n] *= 2/3; }
			lastsample = total * 27/65;
			lastsample -= error*2/3;*/

			sample = wavbuf[i];
			error = sample - (modf(sample/16.0, &gb) * 16.0);

			if (gb < 0.0) gb = 0.0;
			if (gb > 15.0) gb = 15.0;
			if ((i&1) == 0) gbbuf[i/2] = (int)gb*16; else gbbuf[i/2] += (int)gb; // high nybble first
			rawbuf[i] = (int)gb*16;
		}
		fwrite(gbbuf, 1, read/2, fgb);
//		fwrite(rawbuf, 1, read, fraw);
		if (kbhit()) break;
	}

	// update block count
	while (((ftell(fgb)-2) & 15) != 0) fputc(0x88, fgb);
	blocks = (ftell(fgb)-2)/16; // + 1; // +1 !!!
	fseek(fgb, 0, SEEK_SET);
	int l = (blocks & 0xFF) + 1;
	int h = (blocks >> 8) + 1;
	fwrite(&l, 1, 1, fgb);
	fwrite(&h, 1, 1, fgb);

	fclose(fwav);
	fclose(fgb);
//	fclose(fraw);
	return 0;
}